classdef ExampleDiagnosisSystem < DiagnosisSystemClass
    properties
        signalIndices = []; % Which signals are used by classifiers
        anomalyDetector = []; % 1SVM-based fault detector
        faultClassifier = []; % Random Forest-based fault classifier
    end
    methods
        % Constructor
        function obj = ExampleDiagnosisSystem()
            
            % Store which signals this diagnosis system is using
            obj.signalIndices = [2 4 5 6 7 8 9];
        end
        %
        function Initialize(obj)
            % Load model
            if isfile('exampleDiagnosisSystemParameters.mat')
                loadDS = load('exampleDiagnosisSystemParameters.mat', 'anomalyDetector','faultClassifier');
                obj.anomalyDetector = loadDS.anomalyDetector;
                obj.faultClassifier = loadDS.faultClassifier;
                fprintf('Example Diagnosis System loaded.\n');
            else
                fprintf('exampleDiagnosisSystemParameters.mat do not exist. Training Diagnosis System.\n');
                obj.Train()
            end
        end        
        % 
        function [detection, isolation] = Input(obj,sample)

            X = sample(obj.signalIndices);

            % fault detection
            [~,score_detection] = predict(obj.anomalyDetector,X);
            detection = score_detection < 0;
            % fault isolation
            if detection
                [~,score_isolation] = predict(obj.faultClassifier, X);
                % This solution is not identifying unknown fault class so that
                % one is always ranked 0
                isolation = [score_isolation, 0];
            else
                isolation = zeros(1,5);
            end
        end
        %
        function Train(obj)
            % Load training data
            data.wltp_NF        = readtable("../data/trainingdata/wltp_NF.csv");
            data.wltp_f_pic_090 = readtable("../data/trainingdata/wltp_f_pic_090.csv");
            data.wltp_f_pic_110 = readtable("../data/trainingdata/wltp_f_pic_110.csv");
            data.wltp_f_pim_080 = readtable("../data/trainingdata/wltp_f_pim_080.csv");
            data.wltp_f_pim_090 = readtable("../data/trainingdata/wltp_f_pim_090.csv");
            data.wltp_f_waf_105 = readtable("../data/trainingdata/wltp_f_waf_105.csv");
            data.wltp_f_waf_110 = readtable("../data/trainingdata/wltp_f_waf_110.csv");
            data.wltp_f_iml_6mm = readtable("../data/trainingdata/wltp_f_iml_6mm.csv");

            % Set random seed
            rng(1);

            % fault-free data for fault detector
            Xdet = data.wltp_NF(1:2:end, obj.signalIndices);
            ydet = ones(size(Xdet,1),1);

            % train fault detector
            fprintf('Training 1SVM-based fault detector... ');
            anomalyDetector = fitcsvm(Xdet,ydet,'KernelScale','auto','Standardize',true,'OutlierFraction',0.02);
            fprintf('Done!\n');         

            % faulty data for fault classifier
            Xfpic = [data.wltp_f_pic_090(1:2:end, obj.signalIndices);data.wltp_f_pic_110(1:2:end, obj.signalIndices)];
            yfpic = 1*ones(size(Xfpic,1),1);
            Xfpim = [data.wltp_f_pim_080(1:2:end, obj.signalIndices);data.wltp_f_pim_090(1:2:end, obj.signalIndices)];
            yfpim = 2*ones(size(Xfpim,1),1);            
            Xfwaf = [data.wltp_f_waf_105(1:2:end, obj.signalIndices);data.wltp_f_waf_110(1:2:end, obj.signalIndices)];
            yfwaf = 3*ones(size(Xfwaf,1),1);                
            Xfiml = [data.wltp_f_iml_6mm(1:2:end, obj.signalIndices)];
            yfiml = 4*ones(size(Xfiml,1),1);  

            Xisol = [Xfpic;Xfpim;Xfwaf;Xfiml];
            yisol = [yfpic;yfpim;yfwaf;yfiml];

            % train fault classifier
            fprintf('Training Random Forest-based fault classifier with 10 trees... ');
            faultClassifier = TreeBagger(10,Xisol,yisol);
            % Save trained models in file
            fprintf('Done!\n');

            fprintf('Save trained diagnosis system in exampleDiagnosisSystemParameters.mat ...');
            save('exampleDiagnosisSystemParameters.mat', 'anomalyDetector', 'faultClassifier');
            fprintf('Done!\n');
        end
        %
    end
end